import os
import shutil
from unittest import TestCase, mock

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.terraform.modules.module_utils import external_modules_download_path
from checkov.terraform.plan_utils import create_definitions

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestGraphBuilder(TestCase):
    def test_build_graph(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/general_example')

        graph_manager = TerraformGraphManager(db_connector=NetworkxConnector())
        graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir)

        expected_num_of_var_nodes = 3
        expected_num_of_locals_nodes = 1
        expected_num_of_resources_nodes = 1
        expected_num_of_provider_nodes = 1
        vertices_by_block_type = graph.vertices_by_block_type
        self.assertEqual(expected_num_of_var_nodes, len(vertices_by_block_type[BlockType.VARIABLE]))
        self.assertEqual(expected_num_of_locals_nodes, len(vertices_by_block_type[BlockType.LOCALS]))
        self.assertEqual(expected_num_of_resources_nodes, len(vertices_by_block_type[BlockType.RESOURCE]))
        self.assertEqual(expected_num_of_provider_nodes, len(vertices_by_block_type[BlockType.PROVIDER]))

        provider_node = graph.vertices[vertices_by_block_type[BlockType.PROVIDER][0]]
        resource_node = graph.vertices[vertices_by_block_type[BlockType.RESOURCE][0]]
        local_node = graph.vertices[graph.vertices_block_name_map[BlockType.LOCALS]["bucket_name"][0]]

        var_bucket_name_node = None
        var_region_node = None
        var_aws_profile_node = None
        for index in vertices_by_block_type[BlockType.VARIABLE]:
            var_node = graph.vertices[index]
            if var_node.name == 'aws_profile':
                var_aws_profile_node = var_node
            if var_node.name == 'bucket_name':
                var_bucket_name_node = var_node
            if var_node.name == 'region':
                var_region_node = var_node

        self.check_edge(graph, resource_node, local_node, 'bucket')
        self.check_edge(graph, resource_node, provider_node, 'provider')
        self.check_edge(graph, resource_node, var_region_node, 'region')
        self.check_edge(graph, provider_node, var_aws_profile_node, 'profile')
        self.check_edge(graph, local_node, var_bucket_name_node, 'bucket_name')

    def check_edge(self, graph, node_from, node_to, expected_label):
        hashed_from = node_from.get_hash()
        hashed_to = node_to.get_hash()
        matching_edges = []
        for edge in graph.edges:
            if graph.vertices[edge.origin].get_hash() == hashed_from and graph.vertices[edge.dest].get_hash() == hashed_to:
                matching_edges.append(edge)
        self.assertGreater(len(matching_edges), 0,
                           f'expected to find edge from [{node_from.block_type} {node_from.name}] to [{node_to.block_type} {node_to.name}] with label [{expected_label}]')
        if not any(e.label == expected_label for e in matching_edges):
            self.fail(
                f'expected to find edge from [{node_from.block_type} {node_from.name}] to [{node_to.block_type} {node_to.name}] with label [{expected_label}], found edges: {[str(e) for e in matching_edges]}')

    @staticmethod
    def get_vertex_by_name_and_type(local_graph, block_type, name, multiple=False):
        vertices = [local_graph.vertices[i] for i in local_graph.vertices_block_name_map[block_type][name]]
        if multiple:
            return vertices
        return vertices[0]

    def test_update_vertices_configs_deep_nesting(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/variable_rendering/render_deep_nesting')
        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        expected_config = {
            "aws_s3_bucket": {
                "default": {
                    "server_side_encryption_configuration": [
                        {
                            "rule": [
                                {
                                    "apply_server_side_encryption_by_default": [
                                        {"kms_master_key_id": [""], "sse_algorithm": ["AES256"]}
                                    ]
                                }
                            ]
                        }
                    ],
                    "__start_line__": 1,
                    "__end_line__": 10,
                    "__address__": "aws_s3_bucket.default"
                }
            }
        }
        actual_config = local_graph.vertices[local_graph.vertices_by_block_type.get(BlockType.RESOURCE)[0]].config
        self.assertDictEqual(expected_config, actual_config)

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
    def test_build_graph_with_linked_modules(self):
        # see the image to view the expected graph in tests/resources/modules/linked_modules/expected_graph.png
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/linked_modules'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=False)

        vertices_by_block_type = local_graph.vertices_by_block_type

        expected_vertices_num_by_type = {
            BlockType.VARIABLE: 5,
            BlockType.RESOURCE: 5,
            BlockType.OUTPUT: 3,
            BlockType.MODULE: 2,
            BlockType.DATA: 1,
        }

        for block_type, count in expected_vertices_num_by_type.items():
            self.assertEqual(count, len(vertices_by_block_type[block_type]))

        output_this_lambda_func_arn = self.get_vertex_by_name_and_type(local_graph, BlockType.OUTPUT,
                                                                       'this_lambda_function_arn')
        output_this_lambda_func_name = self.get_vertex_by_name_and_type(local_graph, BlockType.OUTPUT,
                                                                        'this_lambda_function_name')
        output_this_s3_bucket_id = self.get_vertex_by_name_and_type(local_graph, BlockType.OUTPUT, 'this_s3_bucket_id')
        resource_aws_lambda_function = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                                        'aws_lambda_function.this')
        resource_aws_s3_bucket_policy = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                                         'aws_s3_bucket_policy.this')
        resource_aws_s3_bucket = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.this')

        self.check_edge(local_graph, node_from=output_this_lambda_func_arn, node_to=resource_aws_lambda_function,
                        expected_label='value')
        self.check_edge(local_graph, node_from=output_this_lambda_func_name, node_to=resource_aws_lambda_function,
                        expected_label='value')
        self.check_edge(local_graph, node_from=output_this_s3_bucket_id, node_to=resource_aws_s3_bucket_policy,
                        expected_label='value')
        self.check_edge(local_graph, node_from=output_this_s3_bucket_id, node_to=resource_aws_s3_bucket,
                        expected_label='value')

    @mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
    def test_build_graph_with_linked_registry_modules(self):
        resources_dir = os.path.realpath(
            os.path.join(TEST_DIRNAME, '../resources/modules/registry_security_group_inner_module'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir,
                                                                                      render_variables=True,
                                                                                      download_external_modules=True)

        outputs_vpcs = self.get_vertex_by_name_and_type(local_graph, BlockType.OUTPUT, 'security_group_vpc_id',
                                                        multiple=True)
        resource_flow_log = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                             'aws_flow_log.related_flow_log')
        resource_security_group_this = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                                        'aws_security_group.this')
        resource_security_group_this_name_prefix = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                                                    'aws_security_group.this_name_prefix')

        output_this_security_group_vpc_id_inner = [o for o in outputs_vpcs if 'http-80' in o.path][0]
        output_this_security_group_vpc_id_outer = [o for o in outputs_vpcs if 'http-80' not in o.path][0]

        self.check_edge(local_graph, node_from=resource_flow_log, node_to=output_this_security_group_vpc_id_inner,
                        expected_label='vpc_id')
        self.check_edge(local_graph, node_from=output_this_security_group_vpc_id_outer,
                        node_to=resource_security_group_this, expected_label='value')
        self.check_edge(local_graph, node_from=output_this_security_group_vpc_id_outer,
                        node_to=resource_security_group_this_name_prefix, expected_label='value')

        # cleanup
        if os.path.exists(os.path.join(resources_dir, external_modules_download_path)):
            shutil.rmtree(os.path.join(resources_dir, external_modules_download_path))

    def test_build_graph_with_deep_nested_edges(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/k8_service'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, tf = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        resource_kubernetes_deployment = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                                          'kubernetes_deployment.bazel_remote_cache')
        locals_name = self.get_vertex_by_name_and_type(local_graph, BlockType.LOCALS, 'name')
        locals_labels = self.get_vertex_by_name_and_type(local_graph, BlockType.LOCALS, 'labels')

        self.check_edge(local_graph, node_from=locals_labels, node_to=locals_name,
                        expected_label="labels.app.kubernetes.io/name")
        self.check_edge(local_graph, node_from=resource_kubernetes_deployment, node_to=locals_name,
                        expected_label="metadata.name")
        self.check_edge(local_graph, node_from=resource_kubernetes_deployment, node_to=locals_name,
                        expected_label="spec.template.metadata.name")
        self.check_edge(local_graph, node_from=resource_kubernetes_deployment, node_to=locals_name,
                        expected_label="spec.template.spec.container.name")
        self.check_edge(local_graph, node_from=resource_kubernetes_deployment, node_to=locals_name,
                        expected_label="spec.template.spec.volume.1.config_map.name")

    def test_blocks_from_local_graph_module(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules/stacks'))
        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, tf = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        tf, _ = convert_graph_vertices_to_tf_definitions(local_graph.vertices, resources_dir)
        found_results = 0
        for key, value in tf.items():
            if key.file_path.startswith(os.path.join(os.path.dirname(resources_dir), 's3_inner_modules', 'inner', 'main.tf')):
                conf = value['resource'][0]['aws_s3_bucket']['inner_s3']
                new_key = build_new_key_for_tf_definition(key)
                if 'stage/main' in new_key or 'prod/main' in new_key:
                    self.assertTrue(conf['versioning'][0]['enabled'][0])
                    found_results += 1
                elif 'test/main' in new_key:
                    self.assertFalse(conf['versioning'][0]['enabled'][0])
                    found_results += 1
        self.assertEqual(found_results, 3)

    def test_build_graph_with_dynamic_blocks(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/dynamic_lambda_function'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, tf = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        lambda_attributes = local_graph.vertices[0].attributes
        self.assertIn("dead_letter_config", lambda_attributes.keys())

    def test_get_attribute_dict_with_list_value(self):
        # given
        resources_dir = os.path.join(TEST_DIRNAME, "../resources/s3_bucket_grant")
        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        # when
        attributes = local_graph.vertices[
            local_graph.vertices_by_block_type.get(BlockType.RESOURCE)[0]
        ].get_attribute_dict()

        # then
        expected_grant_attribute = [
            {"permissions": ["READ_ACP"], "type": "Group", "uri": "http://acs.amazonaws.com/groups/global/AllUsers"},
            {"id": "1234567890", "permissions": ["FULL_CONTROL"], "type": "CanonicalUser"},
        ]

        self.assertCountEqual(expected_grant_attribute, attributes["grant"])

    def test_build_graph_terraform_block(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/terraform_block')

        graph_manager = TerraformGraphManager(db_connector=NetworkxConnector())
        graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir)

        terraform_blocks = graph.vertices_by_block_type[BlockType.TERRAFORM]
        self.assertEqual(1, len(terraform_blocks))

        terraform_block = graph.vertices[terraform_blocks[0]]
        expected_attributes = ["backend", "required_version", "required_providers"]
        for attr in expected_attributes:
            self.assertIn(attr, list(terraform_block.attributes.keys()))

    @mock.patch.dict(os.environ, {"CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES": "True"})
    def test_build_graph_with_cross_variables_connections(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/cross_variables')

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        var_bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket_public_access_block.var_bucket')
        bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.example')

        self.assertEqual(len(local_graph.edges), 4)
        self.check_edge(local_graph, node_from=var_bucket_resource, node_to=bucket_resource,
                        expected_label="[cross-variable] bucket")

    @mock.patch.dict(os.environ, {"CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES": "True"})
    def test_build_graph_with_cross_variables_connections_from_module(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/cross_variables2/main'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        var_bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket_public_access_block.var_bucket')
        bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.example')

        self.assertEqual(len(local_graph.edges), 6)
        self.check_edge(local_graph, node_from=var_bucket_resource, node_to=bucket_resource,
                        expected_label="[cross-variable] bucket")

    @mock.patch.dict(os.environ, {"CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES": "True"})
    def test_build_graph_with_cross_modules_connections(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/cross_modules'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        var_bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                               'aws_s3_bucket_public_access_block.var_bucket')
        bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.example')

        self.assertEqual(len(local_graph.edges), 5)
        self.check_edge(local_graph, node_from=var_bucket_resource, node_to=bucket_resource,
                        expected_label="[cross-variable] bucket")

    @mock.patch.dict(os.environ, {"CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES": "True"})
    def test_build_graph_with_cross_nested_modules_connections(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/cross_modules2'))

        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)

        var_bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE,
                                                               'aws_s3_bucket_public_access_block.var_bucket')
        bucket_resource = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.example')

        self.assertEqual(len(local_graph.edges), 8)
        self.check_edge(local_graph, node_from=var_bucket_resource, node_to=bucket_resource,
                        expected_label="[cross-variable] bucket")

    def test_nested_modules_address_attribute(self):
        resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/nested_modules_address'))
        graph_manager = TerraformGraphManager(NetworkxConnector())
        local_graph, _ = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=True)
        module_1 = self.get_vertex_by_name_and_type(local_graph, BlockType.MODULE, 'inner_s3_module')
        assert module_1.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) == 'module.s3_module.inner_s3_module'
        module_2 = self.get_vertex_by_name_and_type(local_graph, BlockType.MODULE, 's3_module')
        assert module_2.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) == 's3_module'
        resource_1 = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket_public_access_block.var_bucket')
        assert resource_1.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) == 'module.s3_module.module.inner_s3_module.aws_s3_bucket_public_access_block.var_bucket'
        resource_2 = self.get_vertex_by_name_and_type(local_graph, BlockType.RESOURCE, 'aws_s3_bucket.example')
        assert resource_2.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) == 'aws_s3_bucket.example'
        provider = self.get_vertex_by_name_and_type(local_graph, BlockType.PROVIDER, 'aws.test_provider')
        assert provider.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS) == 'aws.test_provider'

    # Related to https://github.com/bridgecrewio/checkov/issues/4324
    def test_build_graph_for_each_with_variables_and_dynamic_not_crash(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/for_each')

        graph_manager = TerraformGraphManager(db_connector=NetworkxConnector())
        # Shouldn't throw exception
        graph_manager.build_graph_from_source_directory(resources_dir)

    def test_build_rustworkx_graph(self):
        resources_dir = os.path.join(TEST_DIRNAME, '../resources/general_example')

        graph_manager = TerraformGraphManager(db_connector=RustworkxConnector())
        graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir)

        expected_num_of_var_nodes = 3
        expected_num_of_locals_nodes = 1
        expected_num_of_resources_nodes = 1
        expected_num_of_provider_nodes = 1
        vertices_by_block_type = graph.vertices_by_block_type
        self.assertEqual(expected_num_of_var_nodes, len(vertices_by_block_type[BlockType.VARIABLE]))
        self.assertEqual(expected_num_of_locals_nodes, len(vertices_by_block_type[BlockType.LOCALS]))
        self.assertEqual(expected_num_of_resources_nodes, len(vertices_by_block_type[BlockType.RESOURCE]))
        self.assertEqual(expected_num_of_provider_nodes, len(vertices_by_block_type[BlockType.PROVIDER]))

        provider_node = graph.vertices[vertices_by_block_type[BlockType.PROVIDER][0]]
        resource_node = graph.vertices[vertices_by_block_type[BlockType.RESOURCE][0]]
        local_node = graph.vertices[graph.vertices_block_name_map[BlockType.LOCALS]["bucket_name"][0]]

        var_bucket_name_node = None
        var_region_node = None
        var_aws_profile_node = None
        for index in vertices_by_block_type[BlockType.VARIABLE]:
            var_node = graph.vertices[index]
            if var_node.name == 'aws_profile':
                var_aws_profile_node = var_node
            if var_node.name == 'bucket_name':
                var_bucket_name_node = var_node
            if var_node.name == 'region':
                var_region_node = var_node

        self.check_edge(graph, resource_node, local_node, 'bucket')
        self.check_edge(graph, resource_node, provider_node, 'provider')
        self.check_edge(graph, resource_node, var_region_node, 'region')
        self.check_edge(graph, provider_node, var_aws_profile_node, 'profile')
        self.check_edge(graph, local_node, var_bucket_name_node, 'bucket_name')

    def test_multiple_nested_module_with_connected_resources(self):
        valid_plan_path = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules_edges_tfplan/tfplan.json'))
        definitions, definitions_raw = create_definitions(root_folder=None, files=[valid_plan_path])
        graph_manager = TerraformGraphManager(db_connector=RustworkxConnector())
        tf_plan_local_graph = graph_manager.build_graph_from_definitions(definitions, render_variables=False)
        self.assertTrue(tf_plan_local_graph.in_edges[1])
        self.assertTrue(tf_plan_local_graph.in_edges[3])

    def test_best_match_multiple_modules_with_connected_resources(self):
        valid_plan_path = os.path.realpath(os.path.join(TEST_DIRNAME, '../resources/modules_edges_tfplan/tfplan.json'))
        definitions, definitions_raw = create_definitions(root_folder=None, files=[valid_plan_path])
        graph_manager = TerraformGraphManager(db_connector=RustworkxConnector())
        tf_plan_local_graph = graph_manager.build_graph_from_definitions(definitions, render_variables=False)
        origin_module_name = 'module.test.test.s3-bucket-1.aws_s3_bucket_public_access_block.this[0]'
        vertex_module_name_1 = 'module.test.test.s3-bucket-1.aws_s3_bucket.this[0]'
        vertex_module_name_2 = 'module.test.s3-bucket-2.aws_s3_bucket.this[0]'
        origin_path = 'modules_edges_tfplan/tfplan.json'
        common_prefix_1 = tf_plan_local_graph._get_common_prefix_name(origin_module_name, vertex_module_name_1, origin_path)
        common_prefix_2 = tf_plan_local_graph._get_common_prefix_name(origin_module_name, vertex_module_name_2, origin_path)
        assert(common_prefix_1 == 'modules_edges_tfplan/tfplan.json module.test.test.s3-bucket-1')
        assert(common_prefix_2 == 'modules_edges_tfplan/tfplan.json module.test')


def build_new_key_for_tf_definition(key):
    key = key.tf_source_modules
    new_key = ''
    while key.nested_tf_module:
        new_key += f'{key.nested_tf_module.path}'
        key = key.nested_tf_module
    return new_key



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.IAMRoleAllowsPublicAssume import check
from checkov.common.models.enums import CheckResult


class TestIAMRoleAllowsPublicAssume(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_iam_role" "lambdaRole" {
    name = "test-role"
    assume_role_policy = <<EOF
{
"Version": "2012-10-17",
"Statement": [
{
"Action": "sts:AssumeRole",
"Principal" : {"Service": "lambda.amazonaws.com"},
"Effect": "Allow"
},
{
"Action": "sts:AssumeRole",
"Principal" : {"AWS": "*"},
"Effect": "Allow"
},
{
"Action": "sts:AssumeRole",
"Principal" : {"Service": "events.amazonaws.com"},
"Effect": "Allow"
}
]
}

EOF
}        
        """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_array(self):
        hcl_res = hcl2.loads("""
resource "aws_iam_role" "lambdaRole" {
    name = "test-role"
    assume_role_policy = <<EOF
{
"Version": "2012-10-17",
"Statement": [
{
"Action": "sts:AssumeRole",
"Principal" : {"Service": "lambda.amazonaws.com"},
"Effect": "Allow"
},
{
"Action": "sts:AssumeRole",
"Principal" : {"AWS": ["*"]},
"Effect": "Allow"
},
{
"Action": "sts:AssumeRole",
"Principal" : {"Service": "events.amazonaws.com"},
"Effect": "Allow"
}
]
}

EOF
}        
        """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_iam_role" "lambdaRole" {
    name = "test-role"
    assume_role_policy = <<EOF
{
"Version": "2012-10-17",
"Statement": [
{
"Action": "sts:AssumeRole",
"Principal" : {"Service": "lambda.amazonaws.com"},
"Effect": "Allow"
}
]
}

EOF
}        
        """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_deny(self):
        hcl_res = hcl2.loads("""
resource "aws_iam_role" "lambdaRole" {
    name = "test-role"
    assume_role_policy = <<EOF
{
"Version": "2012-10-17",
"Statement": [
{
"Action": "sts:AssumeRole",
"Principal" : {"AWS": "*"},
"Effect": "Deny"
}
]
}

EOF
}        
        """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_empty_iam_policy(self):
        hcl_res = hcl2.loads("""
        resource "aws_iam_role" "lambdaRole" {
            name = "test-role"
            assume_role_policy = ""
        }        
                """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_empty_iam_policy_2(self):
        hcl_res = hcl2.loads("""
        resource "aws_iam_role" "lambdaRole" {
            name = "test-role"
        }        
                """)
        resource_conf = hcl_res['resource'][0]['aws_iam_role']['lambdaRole']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path

import pytest

from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType

from checkov.common.util.consts import START_LINE, END_LINE
from checkov.terraform.graph_manager import TerraformGraphManager
from checkov.terraform_json.parser import parse

EXAMPLES_DIR = Path(__file__).parent / "examples"


@pytest.mark.parametrize(
    "graph_connector",
    [
        NetworkxConnector,
        RustworkxConnector,
    ],
)
def test_build_graph_from_definitions(graph_connector):
    # given
    test_file = EXAMPLES_DIR / "cdk.tf.json"
    graph_manager = TerraformGraphManager(db_connector=graph_connector(), source="Terraform")
    template, _ = parse(file_path=test_file)

    # when
    local_graph = graph_manager.build_graph_from_definitions(
        definitions={str(test_file): template}, render_variables=True
    )

    # then
    assert len(local_graph.vertices) == 6

    bucket_idx = local_graph.vertices_block_name_map["resource"]["aws_s3_bucket.bucket"][0]
    bucket = local_graph.vertices[bucket_idx]

    assert bucket.block_type == BlockType.RESOURCE
    assert bucket.id == "aws_s3_bucket.bucket"
    assert bucket.source == "Terraform"
    assert bucket.attributes[CustomAttributes.RESOURCE_TYPE] == ["aws_s3_bucket"]
    assert bucket.attributes[START_LINE] == 46
    assert bucket.attributes[END_LINE] == 65
    assert bucket.config == {
        "aws_s3_bucket": {
            "bucket": {
                "//": {
                    "checkov": {
                        "skip": [
                            {
                                "comment": "Access logging not needed",
                                "id": "CKV_AWS_18",
                                "__startline__": 50,
                                "__endline__": 53,
                            }
                        ],
                        "__startline__": 48,
                        "__endline__": 55,
                    },
                    "metadata": {
                        "path": "AppStack/bucket",
                        "uniqueId": "bucket",
                        "__startline__": 56,
                        "__endline__": 59,
                    },
                    "__startline__": 47,
                    "__endline__": 60,
                },
                "tags": [{"Name": "example", "Private": "true", "__startline__": 61, "__endline__": 64}],
                "__startline__": 46,
                "__endline__": 65,
                "__address__": "aws_s3_bucket.bucket",
            }
        }
    }



import unittest

from checkov.common.bridgecrew.code_categories import CodeCategoryType, CodeCategoryConfiguration
from checkov.common.bridgecrew.integration_features.features.repo_config_integration import \
    RepoConfigIntegration
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.bridgecrew.severities import BcSeverities, Severities


class TestRepoConfigIntegration(unittest.TestCase):

    def test_integration_valid(self):
        instance = BcPlatformIntegration()
        instance.skip_download = False
        instance.platform_integration_configured = True

        repo_config_integration = RepoConfigIntegration(instance)

        self.assertTrue(repo_config_integration.is_valid())

        instance.skip_download = True
        self.assertFalse(repo_config_integration.is_valid())

        instance.platform_integration_configured = False
        self.assertFalse(repo_config_integration.is_valid())

        instance.skip_download = False
        self.assertFalse(repo_config_integration.is_valid())

        repo_config_integration.integration_feature_failures = True
        self.assertFalse(repo_config_integration.is_valid())

    def test_enforcement_rule_default(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '1')

    def test_enforcement_rule_default_non_matching(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "org/other_repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '1')

    def test_enforcement_rule_simple_match(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "org/repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '2')

    def test_enforcement_rule_cli_repo_match(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "bcorg_org/repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '2')

    def test_enforcement_rule_vcs_and_cli_repo_match(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "bcorg_org/repo"
                        },
                        {
                            "accountId": "5678",
                            "accountName": "org/repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '2')

    def test_enforcement_rule_conflicting_match(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "bcorg_org/repo"
                        }
                    ]
                },
                {
                    "id": "3",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule3",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "5678",
                            "accountName": "org/repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '3')

    def test_enforcement_rule_conflicting_multiple_vcs_match(self):
        enforcement_rule_config = {
            "rules": [
                {
                    "id": "1",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "Security default findings",
                    "createdBy": "Bridgecrew",
                    "mainRule": True,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": []
                },
                {
                    "id": "2",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule2",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "1234",
                            "accountName": "org/repo"
                        }
                    ]
                },
                {
                    "id": "3",
                    "creationDate": "2022-05-02T12:18:27.379Z",
                    "name": "rule3",
                    "createdBy": "Bridgecrew",
                    "mainRule": False,
                    "editable": True,
                    "codeCategories": {
                        "LICENSES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "VULNERABILITIES": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "CRITICAL",
                            "commentsBotThreshold": "LOW"
                        },
                        "IAC": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "SECRETS": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        },
                        "BUILD_INTEGRITY": {
                            "softFailThreshold": "LOW",
                            "hardFailThreshold": "LOW",
                            "commentsBotThreshold": "LOW"
                        }
                    },
                    "repositories": [
                        {
                            "accountId": "5678",
                            "accountName": "org/repo"
                        }
                    ]
                },
            ],
            "accountsNotInMainRule": []
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_enforcement_rules(enforcement_rule_config)
        self.assertEqual(repo_config_integration.enforcement_rule['id'], '2')

    def test_enforcement_rule_constants(self):
        # tests to ensure that the correct constants get updated as rules and runners change
        module_keys = [e.value for e in CodeCategoryType]
        self.assertEqual(set(module_keys), {'IAC', 'SECRETS', 'VULNERABILITIES', 'LICENSES', 'BUILD_INTEGRITY', 'WEAKNESSES'})

    def test_global_soft_fail(self):
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.LOW]).is_global_soft_fail())
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.MEDIUM]).is_global_soft_fail())
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.HIGH]).is_global_soft_fail())
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.CRITICAL]).is_global_soft_fail())
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.INFO]).is_global_soft_fail())
        self.assertFalse(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.MODERATE]).is_global_soft_fail())
        self.assertTrue(CodeCategoryConfiguration('', Severities[BcSeverities.LOW], Severities[BcSeverities.OFF]).is_global_soft_fail())

    def test_skip_paths_empty(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [
                            "org/repo"
                        ],
                        "rule": {
                            "excludePaths": []
                        },
                        "isDefault": True
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, set())

    def test_skip_paths_non_empty(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [
                            "org/repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "a/b"
                            ]
                        },
                        "isDefault": True
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, {'a/b'})

    def test_skip_paths_non_matching(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [
                            "org/other_repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "a/b"
                            ]
                        },
                        "isDefault": True
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, set())

    def test_skip_paths_no_repos(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [],
                        "rule": {
                            "excludePaths": []
                        },
                        "isDefault": True
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, set())

    def test_skip_paths_multiple_one_match(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [
                            "org/repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "a/b"
                            ]
                        },
                        "isDefault": True
                    },
                    {
                        "repos": [
                            "org/other_repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "x/y"
                            ]
                        },
                        "isDefault": False
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, {'a/b'})

    def test_skip_paths_multiple_match(self):
        vcs_config = {
            "scannedFiles": {
                "sections": [
                    {
                        "repos": [
                            "org/repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "a/b"
                            ]
                        },
                        "isDefault": True
                    },
                    {
                        "repos": [
                            "bcorg_org/repo"
                        ],
                        "rule": {
                            "excludePaths": [
                                "x/y"
                            ]
                        },
                        "isDefault": False
                    }
                ]
            }
        }

        instance = BcPlatformIntegration()
        instance.repo_id = 'org/repo'
        repo_config_integration = RepoConfigIntegration(instance)
        repo_config_integration._set_exclusion_paths(vcs_config)
        self.assertEqual(repo_config_integration.skip_paths, {'a/b', "x/y"})


if __name__ == '__main__':
    unittest.main()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class RedisCachePublicNetworkAccessEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Cache for Redis disables public network access"
        id = "CKV_AZURE_89"
        supported_resources = ('Microsoft.Cache/redis',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'properties/publicNetworkAccess'

    def get_expected_value(self) -> str:
        return 'Disabled'


check = RedisCachePublicNetworkAccessEnabled()



import unittest

import hcl2

from checkov.terraform.checks.provider.linode.credentials import check
from checkov.common.models.enums import CheckResult


class TestCredentials(unittest.TestCase):
    def test_success(self):
        hcl_res = hcl2.loads(
            """
            provider "linode" {}
            """
        )
        provider_conf = hcl_res["provider"][0]["linode"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads(
            """
            provider "linode" {
                token = "c7680462065ee80d0fef2940784b1af6826f6e0b18586194c5f67c4b40fa7f09"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["linode"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration, bc_integration
from checkov.terraform import TFDefinitionKey
from checkov.terraform.context_parsers.parsers.resource_context_parser import ResourceContextParser
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.AMICopyUsesCMK import check
from checkov.terraform.runner import Runner
from checkov.terraform.tf_parser import TFParser
from tests.terraform.context_parsers.mock_context_parser import MockContextParser

mock_tf_file = os.path.dirname(os.path.realpath(__file__)) + "/mock_tf_files/mock.tf"
mock_definition = (mock_tf_file, {"mock": [{"mock_type": {"mock_name": {"value": ["mock_value"]}}}]})

inline_tf_file = os.path.dirname(os.path.realpath(__file__)) + "/mock_tf_files/inline_suppression.tf"
mock_dir_path = os.path.dirname(os.path.realpath(__file__)) + "/mock_tf_files"


class TestBaseParser(unittest.TestCase):
    def test_enrich_definition_block(self):
        this_integration = BcPlatformIntegration()
        this_integration.get_public_run_config()
        metadata_integration.bc_integration = this_integration
        metadata_integration.pre_scan()
        mock_parser = MockContextParser()
        parser_registry.register(mock_parser)
        definition_context = parser_registry.enrich_definitions_context(mock_definition)
        skipped_checks = definition_context[mock_tf_file]["mock"]["mock_type"]["mock_name"].get("skipped_checks")
        self.assertIsNotNone(skipped_checks)
        self.assertEqual(len(skipped_checks), 3)
        # Ensure checkov IDs are mapped to BC IDs
        self.assertEqual(skipped_checks[2]["id"], "CKV_AWS_15")
        metadata_integration.bc_integration = bc_integration

    def test__compute_definition_end_line_with_multi_curly_brackets(self):
        # given
        mock_parser = MockContextParser()
        mock_parser.filtered_lines = [
            (1, '#data "aws_iam_policy_document" "null" {}'),
            (3, 'resource "aws_subnet" "pub_sub" {'),
            (4, "tags = merge({"),
            (5, 'Name = "${var.network_name}-pub-sub-${element(var.azs, count.index)}"'),
            (6, 'Tier = "public"'),
            (7, '}, var.tags, var.add_eks_tags ? { "kubernetes.io/role/elb" : "1" } : {})'),
            (8, "}"),
        ]
        mock_parser.filtered_line_numbers = [1, 3, 4, 5, 6, 7, 8]

        # when
        end_line_num = mock_parser._compute_definition_end_line(3)

        # then
        self.assertEqual(8, end_line_num)

    def test_inline_suppression(self):
        parser = TFParser()
        _, tf_definition = parser.parse_hcl_module(mock_dir_path,source='TERRAFORM')
        resources_parser = ResourceContextParser()
        parser_registry.register(resources_parser)
        inline_key = TFDefinitionKey(inline_tf_file)
        inline_suppression_definition = tf_definition[inline_key]
        definition_context = parser_registry.enrich_definitions_context((inline_key,inline_suppression_definition))

        aws_s3_bucket_resources = definition_context[inline_key]["resource"]["aws_s3_bucket"]

        self.assertEqual(len(aws_s3_bucket_resources["multi-line-multi-checks"].get("skipped_checks")), 3)
        self.assertEqual(len(aws_s3_bucket_resources["multi-line-no-comment"].get("skipped_checks")), 2)
        self.assertEqual(len(aws_s3_bucket_resources["one-line-one-check"].get("skipped_checks")), 1)
        self.assertEqual(len(aws_s3_bucket_resources["one-line-multi-checks"].get("skipped_checks")), 2)
        self.assertEqual(len(aws_s3_bucket_resources["no-comment"].get("skipped_checks")), 0)


if __name__ == "__main__":
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestJsonpathNotExistsSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super().setUp()

    def test_jsonpath_not_exists_solver_simple(self):
        root_folder = '../../../resources/public_security_groups'
        check_id = "PublicDBSG"
        should_fail = ['aws_security_group.aws_security_group_private']
        should_pass = ['aws_db_security_group.aws_db_security_group_public']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_jsonpath_not_exists_solver_wildcard(self):
        root_folder = '../../../resources/security_group_multiple_rules3'
        check_id = "CkSshPortOpenForAll"
        should_fail = ['aws_security_group.sg1']
        should_pass = ['aws_security_group.sg2', 'aws_security_group.sg3']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_jsonpath_not_exists_azure_rule(self):
        root_folder = '../../../resources/azure_secure_rule'
        check_id = "AzureSecureRule"
        should_fail = ['azurerm_network_security_group.sg_fail']
        should_pass = ['azurerm_network_security_group.sg_fail2']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_jsonpath_not_exists_example(self):
        root_folder = './'
        check_id = "example"
        should_pass = ['xyz.pass']
        should_fail = ['xyz.fail']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import copy
import os
import os.path

from parameterized import parameterized_class

from tests.graph_utils.utils import set_db_connector_by_graph_framework, PARAMETERIZED_GRAPH_FRAMEWORKS
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.kubernetes.graph_manager import KubernetesGraphManager
from tests.common.graph.checks.test_yaml_policies_base import TestYamlPoliciesBase


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestYamlPolicies(TestYamlPoliciesBase):
    def tearDown(self) -> None:
        self.get_checks_registry().checks = []

    def __init__(self, args):
        db_connector = set_db_connector_by_graph_framework(self.graph_framework)
        graph_manager = KubernetesGraphManager(db_connector=db_connector)
        real_graph_checks_relative_path = "checkov/kubernetes/checks/graph_checks"
        real_graph_checks_path = os.path.join(os.path.dirname(__file__), '..', '..', '..', '..',
                                              real_graph_checks_relative_path)
        super().__init__(graph_manager, real_graph_checks_path,
                         os.path.dirname(__file__) + "/test_checks", 'kubernetes', __file__, args)

    def test_AllowedCapabilities(self):
        self.go('AllowedCapabilities')

    def test_AllowPrivilegeEscalation(self):
        self.go('AllowPrivilegeEscalation')

    def test_RoleBindingPE(self) -> None:
        self.go('RoleBindingPE')

    def test_NoCreateNodesProxyOrPodsExec(self) -> None:
        self.go('NoCreateNodesProxyOrPodsExec')
    
    def test_ImpersonatePermissions(self) -> None:
        self.go("ImpersonatePermissions")

    def test_ModifyServicesStatus(self) -> None:
        self.go('ModifyServicesStatus')

    def test_ReadAllSecrets(self) -> None:
        self.go('ReadAllSecrets')

    def test_PodIsPubliclyAccessibleExample(self) -> None:
        self.go('PodIsPubliclyAccessibleExample')

    def test_RequireAllPodsToHaveNetworkPolicy(self) -> None:
        self.go('RequireAllPodsToHaveNetworkPolicy')

    def test_NoDefaultNamespace(self):
        self.go('NoDefaultNamespace')

    def create_report_from_graph_checks_results(self, checks_results, check):
        report = Report("kubernetes")
        first_results_key = list(checks_results.keys())[0]
        for check_result in checks_results[first_results_key]:
            entity = check_result["entity"]
            record = Record(check_id=check['id'],
                            check_name=check['name'],
                            check_result=copy.deepcopy(check_result),
                            code_block="",
                            file_path=entity.get(CustomAttributes.FILE_PATH),
                            file_line_range=[entity.get('__startline__'), entity.get('__endline__')],
                            resource=entity.get(CustomAttributes.BLOCK_NAME),
                            evaluations={},
                            check_class=check.__class__.__module__,
                            file_abs_path=entity.get(CustomAttributes.FILE_PATH))
            if check_result["result"] == CheckResult.PASSED:
                report.passed_checks.append(record)
            if check_result["result"] == CheckResult.FAILED:
                report.failed_checks.append(record)
        return report

    def assert_evaluated_keys(self, checks_results, check):
        pass



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.AzureScaleSetPassword import check
from checkov.common.models.enums import CheckResult


class TestAzureScaleSetPassword(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_linux_virtual_machine_scale_set" "example" {
            name                = var.scaleset_name
            resource_group_name = var.resource_group.name
            location            = var.resource_group.location
            sku                 = var.sku
            instances           = var.instance_count
            admin_username      = var.admin_username
            disable_password_authentication = false
            tags = var.common_tags
        }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_linux_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_linux_virtual_machine_scale_set" "example" {
            name                = var.scaleset_name
            resource_group_name = var.resource_group.name
            location            = var.resource_group.location
            sku                 = var.sku
            instances           = var.instance_count
            admin_username      = var.admin_username
            disable_password_authentication = true

                admin_ssh_key {
                    username   = var.admin_username
                    public_key = tls_private_key.new.public_key_pem
                }
            tags = var.common_tags
        }
                        """)
        resource_conf = hcl_res['resource'][0]['azurerm_linux_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.KeyVaultEnablesSoftDelete import check
from checkov.common.models.enums import CheckResult


class TestKeyVaultEnablesSoftDelete(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "examplekeyvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              soft_delete_retention_days  = 7
              purge_protection_enabled    = false
              soft_delete_enabled = false
              sku_name = "standard"
            
              access_policy {
                tenant_id = data.azurerm_client_config.current.tenant_id
                object_id = data.azurerm_client_config.current.object_id
            
                key_permissions = [
                  "Get",
                ]
            
                secret_permissions = [
                  "Get",
                ]
            
                storage_permissions = [
                  "Get",
                ]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "examplekeyvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              soft_delete_retention_days  = 7
              purge_protection_enabled    = false
            
              sku_name = "standard"
            
              access_policy {
                tenant_id = data.azurerm_client_config.current.tenant_id
                object_id = data.azurerm_client_config.current.object_id
            
                key_permissions = [
                  "Get",
                ]
            
                secret_permissions = [
                  "Get",
                ]
            
                storage_permissions = [
                  "Get",
                ]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_key_vault" "example" {
              name                        = "examplekeyvault"
              location                    = azurerm_resource_group.example.location
              resource_group_name         = azurerm_resource_group.example.name
              enabled_for_disk_encryption = true
              tenant_id                   = data.azurerm_client_config.current.tenant_id
              soft_delete_retention_days  = 7
              purge_protection_enabled    = false
              soft_delete_enabled = true
              sku_name = "standard"
            
              access_policy {
                tenant_id = data.azurerm_client_config.current.tenant_id
                object_id = data.azurerm_client_config.current.object_id
            
                key_permissions = [
                  "Get",
                ]
            
                secret_permissions = [
                  "Get",
                ]
            
                storage_permissions = [
                  "Get",
                ]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_key_vault']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

import concurrent
import hashlib
from typing import Any, Callable, overload, Union, List, Dict
import concurrent.futures

from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import force_int


def calculate_hash(data: Any) -> str:
    sha256 = hashlib.sha256(str(data).encode("utf-8"))
    return sha256.hexdigest()


def join_trimmed_strings(char_to_join: str, str_lst: list[str], num_to_trim: int) -> str:
    return char_to_join.join(str_lst[: len(str_lst) - num_to_trim])


def run_function_multithreaded(
    func: Callable[..., Any], data: list[list[Any]], max_group_size: int, num_of_workers: int | None = None
) -> None:
    groups_of_data = [data[i : i + max_group_size] for i in range(0, len(data), max_group_size)]
    if not num_of_workers:
        num_of_workers = len(groups_of_data)
    if num_of_workers > 0:
        with concurrent.futures.ThreadPoolExecutor(max_workers=num_of_workers) as executor:
            futures = {executor.submit(func, data_group): data_group for data_group in groups_of_data}
            wait_result = concurrent.futures.wait(futures)
            if wait_result.not_done:
                raise Exception(f"failed to perform {func.__name__}")
            for future in futures:
                try:
                    future.result()
                except Exception:
                    raise


def filter_sub_keys(key_list: list[str]) -> list[str]:
    filtered_key_list = []
    for key in key_list:
        if not any(other_key != key and other_key.startswith(key) for other_key in key_list) and is_include_dup_dynamic(key, key_list):
            filtered_key_list.append(key)
    return filtered_key_list


def is_include_dup_dynamic(key: str, list_keys: list[str]) -> bool:
    return f"dynamic.{key.split('.')[0]}" not in list_keys


def adjust_value(element_name: str, value: Any) -> Any:
    """Adjusts the value, if the 'element_name' references a nested key

    Ex:
    element_name = publicKey.keyData
    value = {"keyData": "key-data", "path": "path"}

    returns new_value = "key-data"
    """

    if "." in element_name and isinstance(value, dict):
        key_parts = element_name.split(".")
        new_value = value.get(key_parts[1])

        if new_value is None:
            # couldn't find key in in value object
            return None

        return adjust_value(".".join(key_parts[1:]), new_value)

    return value


def to_list(data: Any) -> list[Any] | dict[str, Any]:
    if isinstance(data, list) and len(data) == 1 and (isinstance(data[0], str) or isinstance(data[0], int)):
        return data
    elif isinstance(data, list):
        return [to_list(x) for x in data]
    elif isinstance(data, dict):
        return {key: to_list(val) for key, val in data.items()}
    else:
        return [data]


@overload
def update_dictionary_attribute(
        config: dict[str, Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> dict[str, Any]:
    ...


@overload
def update_dictionary_attribute(
        config: list[Any], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any]:
    ...


def update_dictionary_attribute(
    config: Union[List[Any], Dict[str, Any]], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> Union[List[Any], Dict[str, Any]]:
    key_parts = key_to_update.split(".")
    if '"' in key_to_update:
        key_parts = join_double_quote_surrounded_dot_split(str_parts=key_parts)

    if isinstance(config, dict) and isinstance(key_parts, list):
        key = key_parts[0]
        inner_config = config.get(key)

        if inner_config is not None:
            if len(key_parts) == 1:
                if isinstance(inner_config, list) and not isinstance(new_value, list):
                    new_value = [new_value]
                config[key] = to_list(new_value) if dynamic_blocks else new_value
                return config
            else:
                config[key] = update_dictionary_attribute(
                    inner_config, ".".join(key_parts[1:]), new_value, dynamic_blocks=dynamic_blocks
                )
        else:
            for key in config:
                config[key] = update_dictionary_attribute(
                    config[key], key_to_update, new_value, dynamic_blocks=dynamic_blocks
                )
    if isinstance(config, list):
        return update_list_attribute(
            config=config,
            key_parts=key_parts,
            key_to_update=key_to_update,
            new_value=new_value,
            dynamic_blocks=dynamic_blocks,
        )
    return config


def update_list_attribute(
    config: list[Any], key_parts: list[str], key_to_update: str, new_value: Any, dynamic_blocks: bool = False
) -> list[Any] | dict[str, Any]:
    """Updates a list attribute in the given config"""

    if not config:
        # happens when we can't correctly evaluate something, because of strange defaults or 'for_each' blocks
        return config

    if len(key_parts) == 1 and len(config) == 1:
        idx = force_int(key_parts[0])
        # Avoid changing the config and cause side effects
        inner_config = pickle_deepcopy(config[0])

        if idx is not None and isinstance(inner_config, list):
            if not inner_config:
                # happens when config = [[]]
                return config

            inner_config[idx] = new_value
            return [inner_config]
    entry_to_update = int(key_parts[0]) if key_parts[0].isnumeric() else -1
    for i, config_value in enumerate(config):
        if entry_to_update == -1:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=key_to_update, new_value=new_value, dynamic_blocks=dynamic_blocks)
        elif entry_to_update == i:
            config[i] = update_dictionary_attribute(config=config_value, key_to_update=".".join(key_parts[1:]), new_value=new_value, dynamic_blocks=dynamic_blocks)

    return config


def join_double_quote_surrounded_dot_split(str_parts: list[str]) -> list[str]:
    """Joins back split strings which enclosed a dot by double quotes

    ex.

    ['google_project_iam_binding', 'role["roles/logging', 'admin"]'] -> ['google_project_iam_binding', 'role["roles/logging.admin"]']

    If someone finds a better solution feel free to replace it!
    """

    new_str_parts = []
    joined_str_parts: list[str] = []
    for part in str_parts:
        if not joined_str_parts:
            if '"' not in part:
                new_str_parts.append(part)
            elif part.count('"') >= 2:
                new_str_parts.append(part)
            else:
                joined_str_parts.append(part)
            continue

        joined_str_parts.append(part)

        if '"' in part:
            new_str_parts.append(".".join(joined_str_parts))
            joined_str_parts = []

    return new_str_parts



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestNotContainsSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestNotContainsSolver, self).setUp()

    def test_public_virtual_machines(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "PublicVMs"
        should_pass = ['aws_default_security_group.default_security_group_closed']
        should_fail = ['aws_default_security_group.default_security_group_open']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_list_cidr_blocks(self):
        root_folder = '../../../resources/security_group_list_cidr_blocks'
        check_id = "PublicSG"
        should_pass = ['aws_security_group.passed_cidr_block', 'aws_security_group.failed_cidr_blocks']
        should_fail = []
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_list_cidr_blocks_specific(self):
        root_folder = '../../../resources/security_group_list_cidr_blocks'
        check_id = "SpecificBlockSG"
        should_pass = ['aws_security_group.failed_cidr_blocks']
        should_fail = ['aws_security_group.passed_cidr_block']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



# flake8: noqa

# skip this file from pylint since it vulnerable by design
import json
import os

stream = os.popen('id')
output = stream.read()
result = {"result": output}

print(json.dumps(result))



from aws_cdk import core
from aws_cdk import aws_ecr as ecr

class MyECRStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an ECR repository with image scanning on push enabled
        ecr_repository = ecr.Repository(
            self,
            "MyECRRepository",
            repository_name="my-ecr-repo",
        )



import pytest

from checkov.gitlab_ci.runner import Runner


@pytest.mark.parametrize(
    "key,file_path,expected_key, start_line, end_line",
    [
        ('*.script[].*.script[].CKV_GITLABCI_1[19:19]', '/checkov/tests/gitlab_ci/resources/images/.gitlab-ci.yml',
         'test.script', 19, 19),
        ('*.rules.*.rules.CKV_GITLABCI_2[7:9]', '/checkov/tests/gitlab_ci/resources/two/.gitlab-ci.yml',
         'planOnlySubset', 7, 9),
    ],
)
def test_get_resource(key, file_path, expected_key, definitions, start_line, end_line):
    runner = Runner()
    runner.definitions = definitions
    new_key = runner.get_resource(file_path, key, [], start_line, end_line)

    assert new_key == expected_key


import unittest
import hcl2

from checkov.terraform.checks.resource.aws.EKSNodeGroupRemoteAccess import check
from checkov.common.models.enums import CheckResult


class TestEKSNodeGroupRemoteAccess(unittest.TestCase):
    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_eks_node_group" "test" {
  cluster_name    = aws_eks_cluster.example.name
  node_group_name = "example"
  node_role_arn   = aws_iam_role.example.arn
  subnet_ids      = aws_subnet.example[*].id

  remote_access {
    ec2_ssh_key = "some-key"
  }

  scaling_config {
    desired_size = 1
    max_size     = 1
    min_size     = 1
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_eks_node_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_eks_node_group" "test" {
  cluster_name    = aws_eks_cluster.example.name
  node_group_name = "example"
  node_role_arn   = aws_iam_role.example.arn
  subnet_ids      = aws_subnet.example[*].id

  remote_access {
    ec2_ssh_key = "some-key"
    source_security_group_ids = "some-group"
  }

  scaling_config {
    desired_size = 1
    max_size     = 1
    min_size     = 1
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_eks_node_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_implicit(self):
        hcl_res = hcl2.loads("""
resource "aws_eks_node_group" "test" {
  cluster_name    = aws_eks_cluster.example.name
  node_group_name = "example"
  node_role_arn   = aws_iam_role.example.arn
  subnet_ids      = aws_subnet.example[*].id

  scaling_config {
    desired_size = 1
    max_size     = 1
    min_size     = 1
  }
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_eks_node_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class AzureBatchAccountEndpointAccessDefaultAction(BaseResourceCheck):

    def __init__(self) -> None:
        name = "Ensure that if Azure Batch account public network access in case 'enabled' then its account access must be 'deny'"
        id = "CKV_AZURE_248"
        supported_resources = ("azurerm_batch_account",)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:

        public_network_access = conf.get('public_network_access_enabled', [None])[0]
        # public network access is disabled, no need to check for account access default action
        if public_network_access is False:
            return CheckResult.PASSED

        network_profile: dict[str, Any] | None = conf.get('network_profile', [None])[0]
        if not network_profile:
            return CheckResult.PASSED
        account_access: dict[str, Any] | None = network_profile.get('account_access', [None])[0]
        if not account_access:
            return CheckResult.PASSED
        default_action: str | None = account_access.get('default_action', [None])[0]
        if not default_action or str(default_action).lower() != "allow":
            return CheckResult.PASSED

        self.evaluated_keys = ["network_profile/[0]/account_access/[0]/default_action"]
        return CheckResult.FAILED


check = AzureBatchAccountEndpointAccessDefaultAction()



import os

from checkov.kubernetes.graph_builder.graph_components.edge_builders.ServiceAccountEdgeBuilder import \
    ServiceAccountEdgeBuilder
from checkov.kubernetes.graph_builder.local_graph import KubernetesLocalGraph
from checkov.kubernetes.parser.parser import parse
from tests.kubernetes.graph.base_graph_tests import TestGraph
from checkov.kubernetes.kubernetes_graph_flags import K8sGraphFlags
from checkov.kubernetes.kubernetes_utils import PARENT_RESOURCE_KEY_NAME, PARENT_RESOURCE_ID_KEY_NAME
from checkov.kubernetes.graph_builder.graph_components.edge_builders.LabelSelectorEdgeBuilder import LabelSelectorEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.edge_builders.KeywordEdgeBuilder import KeywordEdgeBuilder
from checkov.kubernetes.graph_builder.graph_components.edge_builders.NetworkPolicyEdgeBuilder import NetworkPolicyEdgeBuilder

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestKubernetesLocalGraph(TestGraph):
    def test_build_graph_with_single_resource(self) -> None:
        relative_file_path = "../checks/example_AllowedCapabilities/cronjob-PASSED.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        resource = definitions[relative_file_path][0]

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.build_graph(render_variables=False, graph_flags=K8sGraphFlags())
        self.assertEqual(1, len(local_graph.vertices))
        self.assert_vertex(local_graph.vertices[0], resource)

    def test_build_graph_with_multi_resources(self) -> None:
        relative_file_path = "../checks/example_DefaultNamespace/default-k8s-service-and-sa-PASSED2.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        local_graph = KubernetesLocalGraph(definitions)
        local_graph.build_graph(render_variables=False, graph_flags=K8sGraphFlags())
        self.assertEqual(4, len(local_graph.vertices))

    def test_build_graph_with_nested_resources(self) -> None:
        file = os.path.join(TEST_DIRNAME, 'resources', 'nested_resource.yaml')
        definitions = {}
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=False)
        (definitions[file], definitions_raw) = parse(file)
        local_graph = KubernetesLocalGraph(definitions)
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(2, len(local_graph.vertices))
        assert local_graph.vertices[0].id == 'Deployment.default.deployment_name'
        assert local_graph.vertices[0].attributes.get('spec').get('template') is None
        assert local_graph.vertices[0].metadata.name == 'deployment_name'
        assert local_graph.vertices[0].metadata.selector.match_labels.get('app') == 'myapp'
        assert local_graph.vertices[0].metadata.labels is None
        assert local_graph.vertices[1].id == 'Pod.default.deployment_name.app-myapp'
        assert local_graph.vertices[1].config[PARENT_RESOURCE_KEY_NAME] == 'deployment_name'
        assert local_graph.vertices[1].config[PARENT_RESOURCE_ID_KEY_NAME] == 'Deployment.default.deployment_name'
        assert local_graph.vertices[1].config.get('kind') == 'Pod'
        assert local_graph.vertices[1].config.get('apiVersion') == local_graph.vertices[0].config.get('apiVersion')
        assert len(local_graph.vertices[1].attributes.get('spec').get('containers')) == 1
        assert local_graph.vertices[1].metadata.name is None
        assert local_graph.vertices[1].metadata.selector.match_labels is None
        assert local_graph.vertices[1].metadata.labels.get('app') == 'myapp'

    def test_LabelSelectorEdgeBuilder_on_template_with_matched_label_and_selector(self) -> None:
        relative_file_path = "resources/LabelSelector/label_selector_match.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [LabelSelectorEdgeBuilder, ]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(2, len(local_graph.vertices))
        self.assertEqual(1, len(local_graph.edges))

    def test_LabelSelectorEdgeBuilder_on_template_with_non_matched_label_and_selector(self) -> None:
        relative_file_path = "resources/LabelSelector/label_selector_non_match.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [LabelSelectorEdgeBuilder, ]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(2, len(local_graph.vertices))
        self.assertEqual(0, len(local_graph.edges))

    def test_LabelSelectorEdgeBuilder_on_template_with_matched_and_non_matched_label_and_selector(self) -> None:
        relative_file_path = "resources/LabelSelector/label_selector_multiple_resources.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [LabelSelectorEdgeBuilder, ]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(3, len(local_graph.vertices))
        self.assertEqual(1, len(local_graph.edges))

    def test_KeywordEdgeBuilder_on_templates_with_matched_cluster_role_binding(self) -> None:
        relative_file_path = "resources/Keyword/clusterrolebinding.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [KeywordEdgeBuilder, ]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(6, len(local_graph.vertices))
        self.assertEqual(3, len(local_graph.edges))
        self.assertEqual(local_graph.edges[0].origin, 0)
        self.assertEqual(local_graph.edges[0].dest, 3)
        self.assertEqual(local_graph.edges[1].origin, 0)
        self.assertEqual(local_graph.edges[1].dest, 4)

    def test_KeywordEdgeBuilder_and_ServiceAccountEdgeBuilder_on_templates_with_pod_and_service_account(self) -> None:
        relative_file_path = "resources/Keyword/pod_service_account.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [KeywordEdgeBuilder, ServiceAccountEdgeBuilder()]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(4, len(local_graph.vertices))
        self.assertEqual(3, len(local_graph.edges))
        self.assertEqual(local_graph.edges[0].origin, 1)
        self.assertEqual(local_graph.edges[0].dest, 2)
        self.assertEqual(local_graph.edges[1].origin, 1)
        self.assertEqual(local_graph.edges[1].dest, 3)
        self.assertEqual(local_graph.edges[2].origin, 3)
        self.assertEqual(local_graph.edges[2].dest, 0)

    def test_LabelSelectorEdgeBuilder_on_templates_with_network_policy(self) -> None:
        relative_file_path = "resources/Keyword/network-policy-attached.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [NetworkPolicyEdgeBuilder, LabelSelectorEdgeBuilder]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(5, len(local_graph.vertices))
        self.assertEqual(4, len(local_graph.edges))

    def test_extracting_pod_from_container_types(self) -> None:
        relative_file_path = "resources/statefulstate_nested_resource.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [NetworkPolicyEdgeBuilder, LabelSelectorEdgeBuilder]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(2, len(local_graph.vertices))
        self.assertEqual(1, len(local_graph.edges))

    def test_deployment_with_incompatible_selector(self) -> None:
        relative_file_path = "resources/faulty_resources/incompatible_selector.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [NetworkPolicyEdgeBuilder, LabelSelectorEdgeBuilder]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(2, len(local_graph.vertices))
        self.assertEqual(0, len(local_graph.edges))

    def test_KeywordEdgeBuilder_incompatible_cluster_role_binding(self) -> None:
        relative_file_path = "resources/faulty_resources/incompatible_clusterrolebinding.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [KeywordEdgeBuilder, ]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(6, len(local_graph.vertices))
        self.assertEqual(1, len(local_graph.edges))

    def test_deployment_with_missing_metadata(self) -> None:
        relative_file_path = "resources/faulty_resources/deployment_missing_metadata.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [NetworkPolicyEdgeBuilder, LabelSelectorEdgeBuilder]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(0, len(local_graph.vertices))
        self.assertEqual(0, len(local_graph.edges))

    def test_custom_resource_should_not_extract_pod(self) -> None:
        relative_file_path = "resources/custom_resource.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_flags = K8sGraphFlags(create_complex_vertices=True, create_edges=True)

        local_graph = KubernetesLocalGraph(definitions)
        local_graph.edge_builders = [NetworkPolicyEdgeBuilder, LabelSelectorEdgeBuilder]
        local_graph.build_graph(render_variables=False, graph_flags=graph_flags)
        self.assertEqual(1, len(local_graph.vertices))
        self.assertEqual(0, len(local_graph.edges))
